{% extends "base.sh.j2" %}

{# All Available Blocks #}

{# before deploy #}
{# {% block install_commands %} #}
{# {% endblock install_commands %} #}

{# deploy body #}
{# {% block build_commands %} #}
{# {% endblock %} #}

{# post deploy #}
{# {% block finally_build_commands %} #}
{# {% endblock %} #}
